#!/usr/bin/env python
# -*- coding:utf-8 -*-

# DrissionPage 库 文档地址 http://g1879.gitee.io/drissionpagedocs/

#-导入库
from DrissionPage import ChromiumPage,ChromiumOptions

#-配置类
class Config:
    UA_android="Mozilla/5.0 (Linux; Android 11; Pixel 5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.62 Mobile Safari/537.36"
    UA_apple="Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.0 Mobile/15E148 Safari/604.1"
    url='https://gitee.com/g1879/DrissionPage'
    port=7878


#-创建配置对象
co=ChromiumOptions()

#-启动配置
co.set_local_port(Config.port)
co.ignore_certificate_errors(True)

#-创建浏览器
page = ChromiumPage(addr_or_opts=co)
#-创建标签页
tab=page.new_tab()



#-打开网址
tab.get(Config.url)

input('go on ?')
  


#!/usr/bin/env python
# -*- coding:utf-8 -*-
# 用这个代码导入需要的库    # pip install  natsort    PyMuPDF  PySimpleGUI



import os
from threading import Thread
import time
from natsort import natsorted
import fitz  # PyMuPDF
import PySimpleGUI as sg
from natsort import natsorted
from colorama import Fore, init
import time



class Tool:   
  

    @staticmethod
    def convert_images_to_pdf(imgdir, output_pdf_name):
        """
        将指定文件夹内的所有图片转换成一个PDF文件。
        
        参数:
        - imgdir: 存放图片的文件夹路径。
        - output_pdf_name: 输出PDF文件的路径。
        """
        doc = fitz.open()  # 创建一个新的空PDF文档
        imglist = os.listdir(imgdir)  # 获取该文件夹内所有文件的列表
        imglist = natsorted(imglist)  # 使用natsorted模块对文件列表进行自然排序
        imgcount = len(imglist)  # 计算文件夹内文件的数量，即图片的数量

        for i, f in enumerate(imglist):
            img = fitz.open(os.path.join(imgdir, f))  # 以文档的形式打开每张图片
            rect = img[0].rect  # 获取图片的尺寸
            pdfbytes = img.convert_to_pdf()  # 将图片转换成PDF格式的字节流
            img.close()  # 关闭图片文档，释放资源
            imgPDF = fitz.open("pdf", pdfbytes)  # 将字节流作为PDF打开
            page = doc.new_page(width=rect.width, height=rect.height)  # 在新PDF中创建一个与图片尺寸相同的新页面
            page.show_pdf_page(rect, imgPDF, 0)  # 将转换得到的PDF内容填充到新页面中

            print(f"导入PDF {f} {i+1}/{imgcount}...")
            
            # 使用PySimpleGUI显示当前的进度
            if not sg.OneLineProgressMeter("导入PDF进度", i+1, imgcount, 'key '+f, orientation='h'):
                print("用户取消操作")
                break  # 如果用户关闭进度条窗口，则退出循环

        doc.save(output_pdf_name)  # 保存PDF文件
        print(f"PDF文件已保存到 {output_pdf_name}") 

    @staticmethod
    def popup_yes_no(info='是否继续？'):
        return sg.popup_yes_no(info)   
    ##  消息提示框
    @staticmethod
    def popup(info='消息提示框！'):
        return sg.popup(info)   
    
    ##  自动消失
    @staticmethod
    def popup_auto_close(info='自动消失的消息提示框！'):
        return sg.popup_auto_close(info,auto_close_duration=5,button_type='POPUP_BUTTONS_YES_NO') 
      
    ##  
    @staticmethod
    def popup_get_text(info='文本输入框！'):
        return sg.popup_get_text(info)   
    ##
    ##  文件选择对话框
    @staticmethod
    def popup_get_file(info='文件选择对话框：'):
        return sg.popup_get_file(info)   
    ##

    ##  强制跳转新标签页
    @staticmethod
    def click_by_force(ele):
        ele.run_js('this.setAttribute("target", "_blank");')   
        ele.run_js('this.click();')   
    ##
        
    ##  命令行线程进度条
    @staticmethod
    def loading_animation():
        n=0
        def thread_progress(t:Thread):
            nonlocal n    
            while t.is_alive():
                for char in "|/-\\":
                    if n==25:
                        n=0
                    dian='#'*n
                    dian2=dian.ljust(25)
                    
                    print(f" Downloading {char} [{dian2}]", end="\r")
                    time.sleep(0.1)

                n=n+1
                
               

        return thread_progress
    ## 树形 监听
    @staticmethod    
    def packet_tree(packet,to_expand='response'):

        def _tree(obj, last_one=True, body='', level=0):
            if obj is None:
                return 1
            
            obj = obj[0] if isinstance(obj, list) and len(obj) !=0 else obj
            if type(obj).__name__=='CaseInsensitiveDict':
                obj=dict(obj)
                pass


            is_dict = isinstance(obj, (dict,))
            not_dict = not is_dict  

            
            show_len = 150
            list_ele = [i for i in dir(obj) if not i.startswith('_')]  if not_dict else obj.keys()

            length = len(list_ele)
            body_unit = '    ' if last_one else '│   '
            tail = '├───'
            new_body = body + body_unit

            if length > 0:
                new_last_one = False
                
                for idx, attr_name in enumerate(list_ele):
                    if idx == length - 1:
                        tail = '└───'
                        new_last_one = True
                    
                    try:
                        packet_attr = getattr(obj, attr_name) if not_dict else obj[attr_name]
                    except:
                        print(f'{attr_name} {obj}  出错！')
                        continue
                    
                    packet_attr_type = type(packet_attr).__name__
                    if    packet_attr_type=='method':
                        continue

                    value = str(packet_attr).split('\n')[0][:show_len]  

                    # 打印属性信息
                    # attr_name=attr_name.ljust(15)
                    if packet_attr_type != 'builtin_function_or_method' : 
                            if packet_attr_type=='dict':
                                print(f'{new_body}{tail}< {Fore.BLUE}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  ')
                            else:
                                if is_dict:
                                    print(f'{new_body}{tail}# {Fore.GREEN}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  {value}')
                                else:
                                    print(f'{new_body}{tail}< {Fore.BLUE}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  {value}')


        

                    # 递归处理特定属性
                    if attr_name in fields_to_expand or packet_attr_type in types_to_expand :
                        _tree(packet_attr, new_last_one, new_body, level + 1)
                




        init()
        # fields_to_expand=['response', 'request', 'body', 'headers', 'values','get']
        fields_to_expand=['get']
        fields_to_expand.append(to_expand)
        types_to_expand=['dict','list']

        print(f'{Fore.YELLOW}{packet}{Fore.RESET}')
        _tree(packet)  

  



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# 用这个代码导入需要的库    pip install DrissionPage   DataRecorder
#  代码版本低于4.0.0，请升级DP库，至少要4.0.0以上    pip install DrissionPage --upgrade


# 原DP库 使用文档地址  http://drissionpage.cn/


# ----------------导入库----------------
from datetime import datetime
import os,re
import random
import threading
import time

from DrissionPage import ChromiumPage
from DrissionPage import ChromiumOptions
from DrissionPage.common import Keys, Actions
from colorama import Fore, init

# --------------配置类---------------
# from  Config import Config
try:
    from .Config import Config
except:
    print('未找到配置文件，正在匹配另一个配置文件')

    from  Config import Config



# 数据类型判断
from DrissionPage.items import SessionElement
from DrissionPage.items import ChromiumElement
from DrissionPage.items import ShadowRoot
from DrissionPage.items import NoneElement
from DrissionPage.items import ChromiumTab
from DrissionPage.items import WebPageTab
from DrissionPage.items import ChromiumFrame


# -------------浏览器类 ----------
class Browser:

    def __init__(self, browser_path="", config="",plugin_path=''):
        self.browser_path = browser_path  

        self.config_browser(config)
        # 加载插件
        if len(plugin_path)>5:

            subfolders = [f.path for f in os.scandir(plugin_path) if f.is_dir()]
            for f in subfolders:
                self.co.add_extension(f)
                print('加载插件目录 ',f)
            # self.co.add_extension(plugin_path)

        self.page = ChromiumPage(addr_or_opts=self.co)

        self.tabs = []

    def config_browser(self, config):
        self.co = ChromiumOptions()

        if len(self.browser_path)>3:
            self.co.set_browser_path(self.browser_path)
        self.co.set_argument("--hide-crash-restore-bubble")
        co_list = config.split(" ")
        if "多开" in config:
            self.co.auto_port()
        if "无头" in config:
            self.co.headless(True)
        if "无图" in config:
            self.co.no_imgs(True)
        if "无js" in config:
            self.co.no_js(True)
        if "静音" in config:
            self.co.mute(True)

        if "匿名" in config:
            self.co.incognito(True)
        if "安卓" in config:
            self.co.set_user_agent(Config.UA_Android)
        if "苹果" in config:
            self.co.set_user_agent(Config.UA_apple)
        if "忽略证书错误" in config:
            self.co.ignore_certificate_errors(True)
        if "最大化" in config:
            self.co.set_argument("--start-maximized")
        if "最小化" in config:
            self.co.set_argument("--start-minimized")
        for i in co_list:
            if i.startswith("超时"):
                self.co.set_timeouts(base=i[2:])
        for i in co_list:
            if i.startswith("端口"):
                self.co.set_local_port(i[2:])
                print("已连接端口为", i[2:])

        for i in co_list:
            if i.startswith("代理"):
                self.co.set_proxy(i[2:])

    def open(self, url):
        self.tabs.append(self.page.new_tab(url))
        return self

    def ac(self, ele: ChromiumElement):
        return Actions(ele)
    
    def new_page(self, url):
        return NewPage(self.page,url)

    def upload(
        self,
        tag: str,
        file_path: str,
    ):
        """
        上传文件到指定标签页

        参数:
        tag (str): 标签的定位语法
        file_path (str): 上传文件的具体路径

        返回:
        self
        """
        tab = self.newest_page
        tab.set.upload_files(file_path)
        # 定位元素并点击
        tab.ele(tag).click()
        tab.wait.upload_paths_inputted()
        return self

    @property
    def newest_page(self):
        # t=self.page.get_tab[0]
        return self.page.get_tab(self.page.latest_tab)
    def get_tree(self,ele:ChromiumElement):
        Tool.tree(ele)

    @property
    def newest_tab(self):

        return self.page.get_tab(self.page.latest_tab)

    def download_path(self, path):
        self.page.set.download_path(path)
        return self
    def load_js(self, js:str):
        js=Config.current_path()+'\\'+js
        self.newest_page.run_js(js,as_expr=True)
        return self

    def temporary_manual_mode(self):
        self.newest_page.add_ele(outerHTML=Config.add_button)
        self.load_js('Tool.js')
        self.newest_page.run_js('setHoverStyle();')
        while True:
            if not  self.newest_page.ele('#randomButton',timeout=0.2):
                break
            time.sleep(0.2)
        return self
    def god_button(self,name='test',onclick='random()'):
        tab=self.newest_page
        div=tab.ele('#god',timeout=0.3)
        if not div:
            tab.add_ele(outerHTML=Config.god_div)
            tab.add_ele(outerHTML=Config.god_css,insert_to=Config.head)
            div=tab.ele('#god',timeout=0.3)

        out_html=Config.god_button.replace('__按钮',name).replace('__onclick',onclick)
        tab.add_ele(outerHTML=out_html,insert_to=div)
        curr_path=os.path.dirname(os.path.abspath(__file__))
        print(curr_path)
        tab.run_js(curr_path+'\\Tool.js',as_expr=True)

        # tab.run_js('setHoverStyle();')

        return self
    def get_current_file_path(self):
        # 获取当前文件的绝对路径
        current_file = os.path.abspath(__file__)
        # 获取当前文件所在的目录路径
        current_dir = os.path.dirname(current_file)

        return current_dir

    def download(self, url):
        self.page.download(url)
        return self
    


    def show_title(self):
        print(self.tab.title)
        return self

    def max(self):
        self.page.set.window.max()
        return self

    def min(self):
        self.page.set.window.mini()
        return self

    def hide(self):
        self.page.set.window.hide()
        return self

    def show(self):
        self.page.set.window.show()
        return self

    def turn_to_canvas0(self,ele):

        cmd=r"""
    function getShot_by_canvas(element) {
        //加载库
        if (!document.getElementById('html2canvas_id')) {
            var script = document.createElement('script');
            script.src = 'https://cdnjs.cloudflare.com/ajax/libs/html2canvas/1.4.1/html2canvas.min.js';
            script.id = 'html2canvas_id';
            document.body.appendChild(script);
        }

     

    }
    getShot_by_canvas(this);
       
                """ 
        ele.run_js(cmd);   

    def get_shot_by_canvas(self, tab: ChromiumTab, ele: ChromiumElement, name='shot.png',mode=2):
        load_code = r"""
                    function getShot_by_canvas() {
                        //加载库
                        if (!document.getElementById('html2canvas_id')) {
                            var script = document.createElement('script');
                            script.src = 'https://cdnjs.cloudflare.com/ajax/libs/html2canvas/1.4.1/html2canvas.min.js';
                            script.id = 'html2canvas_id';
                            document.body.appendChild(script);
                        }
                    }
                    getShot_by_canvas();
                """
                            # script.src = 'https://cdn.bootcdn.net/ajax/libs/html2canvas/1.4.1/html2canvas.min.js';
        # load_code2 = './js/html2canvas.min.js'
        shot_code = r"""
            //截图
            html2canvas(this).then(function (canvas) {
                var img = canvas.toDataURL("image/png");
                var shot_img = document.createElement('img');
                shot_img.src = img;
                shot_img.id = 'shot_img';
                shot_img.style.visibility = 'hidden';  // 设置 visibility 为 hidden
                document.body.appendChild(shot_img);
            });
            """
        if mode==1:
            tab.run_js(load_code)
        else:
            
            js_file=Config.current_dir+'/js/html2canvas.min.js'
            print(f'正在载入截图js文件:{js_file}')
            tab.run_js(js_file)    

        tab.wait.eles_loaded('#html2canvas_id')
        tab.wait(2)
        ele.run_js(shot_code)
        tab.wait.eles_loaded('#shot_img', timeout=20)
        tab.ele('#shot_img').save(name=name)
        tab.remove_ele('#shot_img')

        
        


    def wait(self, num: int):
        time.sleep(num)
        return self
    def quit(self,close_all=False):

        if close_all:
            if 'y' in  input('当前模式会关闭所有浏览器进程，包括你自己手动打开的浏览器，是否继续？(y/n)?: '):
                Config.close_chrome()
            else:
                print('已取消')
        else:
            self.page.quit()

    def vip_open(self, url, port: int = 0):
        """
        打开vip页面

        :param url: 要访问的页面链接
        :param port: 端口号，默认为0,范围是0-12
        :return: 返回当前对象
        """
        self.page.get(Config.jiekou[port] + url)
        self.page.wait(2)
        if self.page.ele("/html/body"):
            self.page.ele("/html/body").click()

        return self

    def listen_and_download_file(self,tab:ChromiumTab,file_name='123',file_type='.mp3  ',download_path='.'):

        key=file_type.split(' ')
        print(key)

        tab.listen.start(key)
        tab.refresh()   

        # 开始监听
        for data_packet in tab.listen.steps(count=1):

            song_url=data_packet.url
            print(song_url)

        # 下载音乐
        tab.set.download_path(download_path)
        tab.download(song_url,rename=file_name)
        tab.wait(2)
        tab.close()

    def scan_files(self,tab:ChromiumTab,file_type=['.mp3'],count=10,timeout=100):

        def count_up():
            for i in range(count):
                yield i
        c=count_up()    

        url_list=[]

        key=file_type if type(file_type)==list  else file_type.split(' ')
        print(key)

        tab.listen.start(key)
        tab.refresh()   

        # 开始监听
        print(f'正在从{tab.title} 中扫描 {key} 文件 ...........')

        for data_packet in tab.listen.steps(count=count,timeout=timeout):

            file_url=data_packet.url
            print(next(c),' ',file_url)
        return url_list   

    def scan_files2(self,url,file_type=['mp3'],count=10,timeout=100):
        def count_up():
            i=0
            while True:
                i=i+1
                yield str(i).rjust(3)
        c=count_up()

        def  tihuan(string:str):

            for j in  file_type:
                string=string.replace(j,Fore.GREEN+j+Fore.RESET)
            return string 
        def current_time():
            # 获取当前时间
            current_time = datetime.now()

            # 将当前时间格式化为字符串形式
            time_string = current_time.strftime("%H:%M:%S")

            return Fore.BLUE+time_string+Fore.RESET         

        def _scan():   

            # 开始监听

            for data_packet in tab.listen.steps(count=count,timeout=timeout):

                file_url=data_packet.url

                url_list.append(file_url)
                # print(next(c),' ',file_url)
                print(f' {next(c)} {current_time()} {Fore.RED}{data_packet.resourceType.rjust(7)}{Fore.RESET}    {tihuan(file_url)}')

            print('\n 扫描结束 \n')    

        url_list=[]
        tab=self.page.new_tab(url)
        key=file_type if type(file_type)==list  else file_type.split(' ')
        tab.listen.start(targets=key)
        tab.refresh()   

        print(f'正在从{tab.title} 中扫描 {key} 文件 ...........\n')
        file_type=[ '.'+i   for  i  in  file_type ]

        # print(key)
        init()
        threading.Thread(target=_scan).start()  

    def listen_all(self,file_type=['mp3'],count=10,timeout=100):
        def count_up():
            i=0
            while True:
                i=i+1
                yield str(i).rjust(3)
        c=count_up()

        def  tihuan(string:str):

            for j in  file_type:
                string=string.replace(j,Fore.GREEN+j+Fore.RESET)
            return string 
        def current_time():
            # 获取当前时间
            current_time = datetime.now()

            # 将当前时间格式化为字符串形式
            time_string = current_time.strftime("%H:%M:%S")

            return Fore.BLUE+time_string+Fore.RESET         

        def _scan(t:ChromiumTab,key): 
            t.listen.start(targets=key)
            t.refresh()  

            # 开始监听

            for data_packet in t.listen.steps(count=count,timeout=timeout):

                file_url=data_packet.url

                url_list.append(file_url)
                # print(next(c),' ',file_url)
                print(f' {next(c)} {current_time()} {Fore.RED}{data_packet.resourceType.rjust(7)}{Fore.RESET}  {tihuan(file_url)} from {Fore.YELLOW+t.url+Fore.RESET}')

            print('\n 扫描结束 \n')    

        init()
        url_list=[]
        tabs=self.page.get_tabs()

        file_type=[ '.'+i   for  i  in  file_type ]
        key=file_type if type(file_type)==list  else file_type.split(' ')

        print(f'正在扫描 {key} 文件 ...........\n')
        for tt in tabs:
            threading.Thread(target=_scan,args=(tt,key)).start()
            time.sleep(1)

    def load_html2canvas(self):
        js_file=Config.current_dir+'/js/html2canvas.min.js'
        print(f'正在载入js文件:{js_file}')
        self.newest_tab.run_js(js_file)

    def get_screenshot_by_js(self,ele:ChromiumElement,rename='shot.png'):

        shot_code = '''
                html2canvas(this).then(function(canvas) {
                        var img = canvas.toDataURL("image/png");
                        var link = document.createElement('a');
                        link.download = 'screenshot.png';
                        link.href = img;
                        link.id='img_shot';
                        link.click();
                        return img;
                    });

        '''.replace('screenshot.png',rename)

        try:
            ele.run_js(shot_code)
        except:
            self.load_html2canvas()
            # self.newest_tab.ele(f'#su').run_js(shot_code)

            ele.run_js(shot_code)

    def get_screenshot_by_js3(self,ele:ChromiumElement,rename='shot.png'):

        shot_code = '''
                    var canvas = html2canvas(this);
                    var img = canvas.toDataURL("image/png");
                    var link = document.createElement('a');
                    link.download = 'screenshot.png';
                    return img;
  

        '''.replace('screenshot.png',rename)

        try:
            ca=ele.run_js(shot_code)
            print(ca)
            self.page.download(ca)
        except:
            self.load_html2canvas()
            ca=ele.run_js(shot_code)
            print(ca)
            self.page.download(ca)

    def get_screenshot_by_js2(self,ele:ChromiumElement,rename='shot.png'):

        shot_code = '''
                            
function captureElementAndInsertImage(element) {
  html2canvas(element).then(function(canvas) {
    var dataUrl = canvas.toDataURL();
    var img = document.createElement('img');
    img.src = dataUrl;
    img.id = 'shot';
    document.body.appendChild(img);
  }).catch(function(error) {
    console.error('截图失败:', error);
  });
}



captureElementAndInsertImage(this);           


           
        '''

        try:
            ele.run_js(shot_code)
            self.page.latest_tab.wait.ele_loaded('#shot')
            self.page.latest_tab.ele('#shot').click.to_download(rename=rename)

        except:
            self.load_html2canvas()
            ele.run_js(shot_code)
            # self.page.latest_tab.wait.ele_loaded('#shot')
            self.page.wait(6)
            print(self.page.ele('#shot'))
            self.page.ele('#shot').click.to_download(rename=rename)

    def download_Blob(self,ele:ChromiumElement):

        shot_code = '''
                            
// 获取视频的 Blob URL
var blobUrl = this.src;
alert(blobUrl);

// 使用 Fetch API 获取 Blob 数据
fetch(blobUrl)
    .then(response => response.blob())
    .then(blobData => {
        // 创建一个临时链接
        var downloadLink = document.createElement("a");
        downloadLink.href = URL.createObjectURL(blobData);
        downloadLink.download = "video.ts"; // 下载的文件名

        // 触发点击下载链接的操作
        downloadLink.click();
    })
    .catch(error => {
        console.error("下载视频时出错：", error);
        alert("下载视频时出错：", error);
    });
           
        '''

        ele.run_js(shot_code)
        self.page.wait(10)

    @property
    def jquery(self):
        return Jquery(self)

    def elements(self, k: str, v: str):
        ele = self.tab.eles(f"@{k}={v}")
        return ele

    @staticmethod
    def read_file(file_name):
        with open(file_name, "r", encoding="utf-8") as file:
            content = file.read()
        return content

    def run(self, script_file: str):
        _page = self.newest_page
        _page.run_js(Browser.read_file(script_file))

    def loadjQuery(self):
        if self.newest_page.ele("#jq", timeout=0.2):
            print("jQuery 已经加载")
        else:
            self.newest_page.run_js(r'.\LoadJquery.js')
            print("jQuery 成功加载入页面...")

    def tab(self,n=0):  # 返回最新的标签页
        return self.page.get_tab(n)

    def download_all_img(self, tag):
        """
        从给定的标签中下载所有图片。

        参数:
        self: 当前对象
        tag: 包含要下载图片的标签对象

        返回值:
        self: 返回当前对象
        """
        for i in tag.eles("t:img"):
            for j in ["png", "jpg", "jpeg", "webp", "gif", "tiff"]:
                if j in i.link:
                    self.page.download(i.link)

        return self

    def get_plugin_list(self):
        tab=self.page.new_tab('chrome://extensions/')

        plugin_list=[]
        for plugin  in  tab.ele('t:extensions-manager').sr('#container').ele('t:cr-view-manager').ele('t:extensions-item-list').sr('t:div@@class=items-container').eles('t:extensions-item'):
            ss=plugin.sr('#content').text
            plugin_list.append(ss)
        return plugin_list  

    def get_bookmarks_list(self):
        tab=self.page.new_tab('chrome://bookmarks/')

        bookmarks_list=[]
        for shu_qian  in  tab.ele('t:bookmarks-app').sr('tag:bookmarks-list').sr('#list').eles('tag:bookmarks-item'):
            ss=shu_qian.sr('#website-text').text
            bookmarks_list.append(ss)
        return bookmarks_list    

# ----------------------截图类


# ----------------jQuery 类----------------
class Jquery:
    cmd2 = """

                    $('.m_slider').css({ 'left': '266px', 'pointer-events': 'none' });
                    $('.flx_v_sildbobx').css({ 'background-color': '#EAFCE4', });
                    $('.m_t_txt').css('display', 'none');
                    $('.m_t_txt_sucess').css('display', 'inline-block');
                    sliderFlag = true;
                    """

    def __init__(self, browser: Browser):
        self.b = browser
        self.cmd = r"""
                    function loadjQuery() {
                  // 创建一个 script 元素
                  var script = document.createElement('script');
                
                  // 设置 script 元素的 src 属性为 jQuery 的 CDN 地址
                  script.src = 'https://code.jquery.com/jquery-3.6.0.min.js';
                  script.id = 'jq';
                
                  // 将 script 元素添加到文档的头部或 body 中
                  document.head.appendChild(script);
                  // 或者使用 document.body.appendChild(script);
                }
                loadjQuery();
                """

        self.load_jquery()

    def load_jquery(self):
        if self.b.newest_page.ele("#jq"):
            print("jQuery 已经加载")
        else:
            self.run(self.cmd)
            print("jQuery 成功加载入页面...")

    def run(self, js_str: str):
        self.b.newest_page.run_js(js_str)
        return self

    def exe(self, js_str: str):  # 有返回值
        return self.b.newest_page.run_js(js_str)


# ------------工具类----------
class Tool:
    @staticmethod
    def pause(t:ChromiumTab):
        t.run_js(Config.eles_code)
        input('当前调试程序暂停中，按任意键继续...')  



    @staticmethod
    def get_json_by_js_fetch(tab:ChromiumTab,fetch_code:str,show_result=False):

        if fetch_code.startswith('\n'):
            fetch_code=fetch_code[1:]
        js_code='''
        async function fetchData() {
            try {
                const response = await fetch_code
                const data = await response.json();// 返回响应对象的json数据
                console.log(data); 

                return data; // 返回数据
            } catch (error) {
                console.error('获取数据时出错:', error);    
                throw error; // 如果有必要，重新抛出错误
            }
        }

        // 调用函数
        return  fetchData();

        '''.replace('fetch_code',fetch_code)

        json_data=tab.run_js(js_code)
        if show_result:
            print(json_data)
        return json_data
    
    @staticmethod
    def get_random_element(input_list) -> ChromiumElement:
        """
        从输入列表中随机选择一个元素并返回。

        :param input_list: 输入的列表
        :return: 随机选择的元素
        """
        if not input_list:
            return None  # 返回 None，如果列表为空

        return random.choice(input_list)

    @staticmethod
    def sniff_and_download_video(
        page, kw: str = 'x://*[@id="post"]/article/div[3]/div'
    ):
        player = page.ele(kw)
        page.actions.wait(2).click(player)
        player.drag(50, 50, 2)
        player.click.at(40, 70)
        print("视频下载中.......")

    @staticmethod
    def sniff_and_download_videos(ele: ChromiumElement):
        # 执行嗅探并下载视频
        player = ele  # 定义变量player，表示ChromiumElement对象

        time.sleep(2)  # 等待2秒
        player.drag(50, 50, 2)  # 拖动player对象到坐标(50, 50)并移动2像素
        player.click.at(40, 70)  # 在坐标(40, 70)处单击player对象
        print("视频下载中.......")  # 打印提示信息"视频下载中......."

    @staticmethod
    def download_img(page):
        for picture in page("@itemprop=articleBody").eles("t:img"):
            picture.save(path=page.title)
            print("saving the picture..." + str(picture.tag))

    @staticmethod
    def click_next(page):
        page.ele("text:下一篇").click()
        time.sleep(3)

    @staticmethod
    def screenshot(ele, name="viewer"):
        ele.get_screenshot(name=name, scroll_to_center=True)

    @staticmethod
    def trees(ele_or_page):
        """把页面或元素对象DOM结构打印出来
        :param ele_or_page: 页面或元素对象
        :return: None
        """
        def _tree(obj, last_one=True, body=''):
            list_ele = obj.children()
            length = len(list_ele)
            body_unit = '    ' if last_one else '│   '
            tail = '├───'
            new_body = body + body_unit

            if length > 0:
                new_last_one = False
                for i in range(length):
                    if i == length - 1:
                        tail = '└───'
                        new_last_one = True
                    e = list_ele[i]

                    attrs = ' '.join([f"{k}='{v}'" for k, v in e.attrs.items()])
                    print(f'{new_body}{tail}<{e.tag} {attrs}>'.replace('\n', ' '))

                    _tree(e, new_last_one, new_body)

        ele = ele_or_page.s_ele()
        attrs = ' '.join([f"{k}='{v}'" for k, v in ele.attrs.items()])
        print(f'<{ele.tag} {attrs}>'.replace('\n', ' '))
        _tree(ele)
    

    @staticmethod
    def tree(ele):
        init()
        e = ele
        print(f"{Fore.BLUE}{Fore.CYAN}<{e.tag}>  {Fore.RESET}{e.attrs}")
        Tool.__tree(e)

    @staticmethod
    def __tree(ele: any, layer=7, has_next_brother=True, body=""):
        if ele.tag == "iframe":
            # ele = page.get_frame(ele)
            ele = ele("x:/html")
            # print(ele.html)
            # print(ele.children())
        try:
            list_ele = ele.children(timeout=0.1)
        except:
            list_ele = []
            print(ele)
            print("无法获取该元素子元素")

        length = len(list_ele)
        body_unit = "│   " if has_next_brother else "    "
        tail = "├───"
        new_body = body + body_unit

        if length > 0 and layer >= 1:
            has_next_brother2 = True
            for i in range(length):
                if i == length - 1:
                    tail = "└───"
                    has_next_brother2 = False
                e = list_ele[i]
                all_body = f"{Fore.BLUE}{new_body}{tail}{Fore.RESET}"

                print(f"{all_body}{Fore.CYAN}<{e.tag}>{Fore.RESET} ")
                Tool.tree_attr(e, all_body, has_next_brother2, layer)

                Tool.__tree(e, layer - 1, has_next_brother2, new_body)

    @staticmethod
    def tree_attr(ele, body, has_next_brother=True, layer=3):
        e: dict = ele.attrs
        has_child = True if ele.tag == "iframe" or ele.child(timeout=0.2) else False

        if layer == 1:
            has_child = False

        part1 = "│" if has_next_brother else " "
        part2 = "│" if has_child else " "
        replace_part = part1 + "   " + part2
        new_body = body.replace("├───", replace_part).replace("└───", replace_part)

        text = "" if ele.tag == "iframe" else ele.text.split("\n")[0]
        if len(text) >= 1:
            e["inner_txt"] = text if len(text) < 150 else text[0:150] + "......"

        if len(e) > 0:
            e["xpath"] = ele.xpath

            max_k_len = max([len(key) for key in e.keys()])
            head = "┌" + "─" * max_k_len + "┐"
            tail = "└" + "─" * max_k_len + "┘"
            print(new_body, head)

            for k, v in e.items():
                key = Fore.GREEN + str(k).ljust(max_k_len) + Fore.RESET + "│"
                content = f"{key}: {v}"

                print(new_body, "│" + key, v)

            print(new_body, tail)


# -----------动作链类----------------------------
class Actions:
    def __init__(self, ele: ChromiumElement) -> None:
        self.e = ele
        pass

    def go_right(self, kw, duration=2, mode="0"):
        if "%" in kw:
            for i in range(20):
                self.e.run_js(
                    rf' this.style.left="{i*5*self.__percentage_to_float(kw)}%" '
                )
                time.sleep(duration / 20)
        else:
            for i in range(20):
                self.e.run_js(rf' this.style.left="{int(kw)/20*i}px" ')
                time.sleep(duration / 20)

        return self

    def go_left(self, kw, duration=2):
        if "%" in kw:
            for i in range(20):
                self.e.run_js(
                    rf' this.style.right="{i*5*self.__percentage_to_float(kw)}%" '
                )
                time.sleep(duration / 20)
        else:
            for i in range(20):
                self.e.run_js(rf' this.style.right="{int(kw)/20*i}px" ')
                time.sleep(duration / 20)

        return self

    def __percentage_to_float(self, percentage):
        value = float(percentage.strip("%")) / 100
        return value


# 工具类

class Use:
    @staticmethod
    def extract_text(s):
        # 直接使用正则表达式提取并返回结果
        return ''.join(re.findall(r'(?<=>)(.+?)(?=<)', s))

    @staticmethod
    def extract_attrs_value(input_string):
        # 直接返回匹配结果
        return re.findall(r'"[^"]+"', input_string)

    @staticmethod
    def extract_attrs_name(input_string):
        # 改进正则表达式以更精确地匹配属性名
        return re.findall(r'\b\w+(?==")', input_string)

    @staticmethod
    def extract_innertext(input_string):
        # 使用正则表达式简化内部文本提取
        match = re.search(r'>(.*?)<', input_string)
        return match.group(1) if match else ''

    @staticmethod
    def raw(input_str):
        input_str = input_str.strip()
        tag_name_match = re.match(r'<(\w+)', input_str)
        tag_name = tag_name_match.group(1) if tag_name_match else ''
        
        tag_attr_values = Use.extract_attrs_value(input_str)
        tag_attr_names = Use.extract_attrs_name(input_str)
        
        attr_all = ''.join([f'@@{name}={value}' for name, value in zip(tag_attr_names, tag_attr_values)])
        attr_all = attr_all.replace('"', '')
        
        txt = Use.extract_text(input_str)
        tag_txt = f'@@text()={txt}' if txt else ''
        
        transformed_str = f'tag:{tag_name}{attr_all}{tag_txt}'
        print(transformed_str)
        return transformed_str
    


class NewPage:
    def __init__(self,browser:ChromiumPage,url):
        self.url = url 
        self.browser=browser
        self.tab:ChromiumTab=None

        

    def __enter__(self):
        self.tab = self.browser.new_tab(self.url)
        return self.tab

    def __exit__(self, exc_type, exc_value, traceback):
        if self.tab:
            # print("关闭页面")
            self.tab.close()





# ---------------- 下面的是测试代码-------------------
if __name__ == '__main__':
    browser=Browser()
    with  browser.new_page('https://tencent.com/')  as t:
        print(t.title)



    input('press any  to continue...')    

    





 

  
 



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# 用这个代码导入需要的库    pip install DrissionPage tabulate  DataRecorder
#  代码版本低于4.0.0，请升级DP库，至少要4.0.0以上    pip install DrissionPage --upgrade


# 原DP库 使用文档地址 http://g1879.gitee.io/drissionpagedocs/whatsnew/4_0/
# 骚神库网址 https://gitee.com/haiyang0726/SaossionPage

import os
import psutil
import platform


class Config:
    body = "x:/html/body"
    head = "x:/html/head"
    Chrome_path = r"C:\Users\Administrator\AppData\Local\Google\Chrome\Application\chrome.exe"
    twinkstar_path = r"C:\Program Files\Twinkstar Browser\twinkstar.exe"
    UA_Android="Mozilla/5.0 (Linux; Android 11; Pixel 5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.62 Mobile Safari/537.36"
    UA_apple="Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.0 Mobile/15E148 Safari/604.1"




    jiekou = [
            "https://www.ckplayer.vip/jiexi/?url=",
            "https://jx.yparse.com/index.php?url=",
            "https://www.8090g.cn/?url=",
            "https://www.ckplayer.vip/jiexi/?url=",
            "https://jx.qqwtt.com/?url=",
            "https://www.pouyun.com/?url=",
            "https://jx.m3u8.tv/jiexi/?url=",
            "https://z1.m1907.top/?jx=",
            "https://www.8090.la/8090/?url=",
            "https://www.pangujiexi.com/jiexi/?url=",
            "https://dmjx.m3u8.tv/?url=",
            "https://vip.bljiex.com/?v=",
            "https://www.mtosz.com/m3u8.php?url=",
            "https://www.playm3u8.cn/jiexi.php?url=",
            "https://www.yemu.xyz/?url=",
            "https://jx.m3u8.tv/jiexi/?url=",
            "https://api.qianqi.net/vip/?url=",
            "https://jx.playerjy.com/?url=",
            "https://jx.we-vip.com/?url=",
            "https://www.8090g.cn/jiexi/?url=",
            "https://vip.mpos.ren/v/?url=",
            "https://movie.heheda.top/?v=",
            "http://vip.wandhi.com/?v=",
            "https://jx.jsonplayer.com/player/?url=",
            "https://jx.playerjy.com/?url=",
            "https://jx.xmflv.com/?url=",
            "https://jx.xmflv.cc/?url=",
            "https://jx.yparse.com/index.php?url=",
            "https://im1907.top/?jx=",
            "https://www.8090g.cn/?url=",
            "https://api.qianqi.net/vip/?url=",
            "https://jx.yangtu.top/?url=",
            "https://www.ckplayer.vip/jiexi/?url=",
        ]
    

    

    add_button = """
                <div id="ice" style="position: fixed; bottom: 20%; left: 0%;">
            <button id="randomButton" onclick="updateRandomUrl()" style="background-color: rgba(223, 135, 20, 0.521); border-radius: 10px 0 0 10px; font-size: larger;">
                关<br>闭<br>手<br>动
            </button>
        </div>

        """
    god_div = """
                <div id="god_father" style="position: fixed; bottom: 35%; right: 0%;">
                    <div id="god" style="display: flex; flex-direction: column-reverse;">
                    </div>

                </div>
        """
    god_css = """
                <style>
                    .god_class {
                        background-color: rgba(127, 106, 219, 0.6);
                        border: none;
                        border-radius: 5px;
                        color: #fff;
                        cursor: pointer;
                        display: inline-block;
                        font-size: larger;
                        font-weight: bold;
                        margin-bottom: 10px;
                        padding: 10px 20px;
                        text-align: center;
                        text-decoration: none;
                        transition: background-color 0.3s ease;
                    }

                    .god_class:hover {
                        background-color: rgba(127, 106, 219, 1);
                    }
                </style>
        """
    god_button = """               

                    <button class="god_class" onclick="__onclick" >
                        __按钮
                    </button>                
        """
    god_button2 = """
                <div id="ice2" style="position: fixed; bottom: 30%; right: 0%;">

                    <button id="randomButton2" onclick="updateRandomUrl2()" style="background-color: rgba(223, 135, 20, 0.301); 
                    border-radius: 5px 0 0 5px; font-size: larger; display: block; margin-bottom: 10px;">
                        万能按钮
                    </button>

                    <button id="randomButton2" onclick="updateRandomUrl2()" style="background-color: rgba(223, 135, 20, 0.301); 
                    border-radius: 5px 0 0 5px; font-size: larger; display: block; margin-bottom: 10px;">
                        万能按钮
                    </button>



                </div>
        """
    myJS = """
        <script>
        function loadjQuery() {
            // 创建一个 script 元素
            var script = document.createElement('script');

            // 设置 script 元素的 src 属性为 jQuery 的 CDN 地址
            script.src = 'https://code.jquery.com/jquery-3.6.0.min.js';
            script.id = 'jq';

            // 将 script 元素添加到文档的头部或 body 中
            document.head.appendChild(script);
            // 或者使用 document.body.appendChild(script);
        }

        function makeTextRed() {
            // 获取页面上的所有文本元素
            var textElements = document.getElementsByTagName('p'); // 获取所有 <p> 标签的文本
            textElements = Array.from(textElements).concat(Array.from(document.getElementsByTagName('span'))); // 获取所有 <span> 标签的文本并合并
        
            // 将所有文本元素的颜色设置为红色
            textElements.forEach(function(element) {
            element.style.color = 'red';
            });
        }
        </script>
        """
    myJS2 = """
        
        function loadjQuery() {
            // 创建一个 script 元素
            var script = document.createElement('script');

            // 设置 script 元素的 src 属性为 jQuery 的 CDN 地址
            script.src = 'https://code.jquery.com/jquery-3.6.0.min.js';
            script.id = 'jq';

            // 将 script 元素添加到文档的头部或 body 中
            document.head.appendChild(script);
            // 或者使用 document.body.appendChild(script);
        }

        function makeTextRed(cc) {
            // 获取页面上的所有文本元素
            var textElements = document.getElementsByTagName('p'); // 获取所有 <p> 标签的文本
            textElements = Array.from(textElements).concat(Array.from(document.getElementsByTagName('span'))); // 获取所有 <span> 标签的文本并合并
        
            // 将所有文本元素的颜色设置为红色
            textElements.forEach(function(element) {
            element.style.color = cc;
            });
        }
        
        """
    
    eles_code=r"""


function eles(dp_yufa,debugMode=false) {

    // 将DP语法解析为JSON对象
    function parseStringToJson(str) {
        // 首先按'@@'分割字符串，得到key=value对
        let pairs = str.split('@@');
    
        // 创建一个空对象
        let result = {};
    
        // 遍历每一个key=value对
        pairs.forEach(pair => {
            // 按'='分割key和value
            let[key,value] = pair.split('=');
            if (key && value) {
                result[key] = value;
            }
        }
        );
    
        return result;
    }
    
    const logo='🦉';
    dp_yufa = dp_yufa.replace('t:', 't=').replace('tag:', 't=');
    let attrs_json = parseStringToJson(dp_yufa)
    var target =  Array.from(document.querySelectorAll('*'));

    if(debugMode) console.log(logo,attrs_json);

    for (let key in attrs_json) {
        let k = key;
        let v = attrs_json[key];
        if(debugMode) console.log(logo,k, v);

        switch (k) {
        case 't':
            target = Array.from(document.querySelectorAll(attrs_json['t']));
            break;

        case 'class':
            v = v.split(' ');
            v.forEach( (cls) => {
                target = target.filter(ele => ele.classList && ele.classList.contains(cls));
            }
            );
            
            break;

        case 'tx()':
            target = target.filter(ele => ele.innerText == v);
            
            break;
        case 'text()':
            target = target.filter(ele => ele.innerText == v);
            
            break;
        default:
            target = target.filter(ele => ele[k] == v);
            
            break;
        }
        
        if(debugMode) console.log(logo,target);
        
    }

    // 输出结果
    if (target.length > 0) {
        for (let i in target) {
            console.log(i, target[i])
        }
    } else {
        console.log('没有匹配的元素')
    }
    // return target;
}

window.eles = eles;

"""
    

    @staticmethod    
    
    def current_path()-> str:
        # 获取当前文件的绝对路径
        current_file = os.path.abspath(__file__)
        # 获取当前文件所在的目录路径
        current_dir = os.path.dirname(current_file)
        
        return str(current_dir)
    
    current_dir=current_path()

    @staticmethod    

    def find_chrome_path():
        drive, path = os.path.splitdrive("C:\\")
        search_dir = os.path.join(drive, os.sep, "Program Files", "Google", "Chrome")
        if not os.path.isdir(search_dir):
            search_dir = os.path.join(drive, os.sep, "Program Files (x86)", "Google", "Chrome")
        for root, dirs, files in os.walk(search_dir):
            if "chrome.exe" in files:
                path_chrome = os.path.join(root, "chrome.exe")
                print('找到谷歌浏览器  '+path_chrome)
                return path_chrome
        return False
    
    @staticmethod
    def kill_chrome_processes():
        chrome_processes = [process for process in psutil.process_iter(attrs=['pid', 'name']) if 'chrome.exe' in process.info['name']]
        
        if chrome_processes:
            for process in chrome_processes:
                pid = process.info['pid']
                os.system(f"taskkill /F /PID {pid}")
            print("成功关闭所有谷歌浏览器进程")
        else:
            print("未找到任何谷歌浏览器进程")
    import os


    @staticmethod
    def kill_chrome_processes2():
        system_platform = platform.system()
        
        if system_platform == 'Windows':
            chrome_processes = [process for process in psutil.process_iter(attrs=['pid', 'name']) if 'chrome.exe' in process.info['name']]
            
            if chrome_processes:
                for process in chrome_processes:
                    pid = process.info['pid']
                    os.system(f"taskkill /F /PID {pid}")
                return "成功关闭谷歌浏览器进程"
            else:
                return "未找到谷歌浏览器进程"
        
        elif system_platform == 'Linux' or system_platform == 'Darwin':
            chrome_processes = [process for process in psutil.process_iter(attrs=['pid', 'name']) if 'chrome' in process.info['name']]
            
            if chrome_processes:
                for process in chrome_processes:
                    pid = process.info['pid']
                    os.system(f"kill -9 {pid}")
                return "成功关闭谷歌浏览器进程"
            else:
                return "未找到谷歌浏览器进程"
        
        else:
            return "不支持的操作系统平台"
        
    @staticmethod
    def close_chrome():
        system = platform.system()
        
        if system == "Windows":
            process_names = ["chrome.exe"]
        elif system == "Linux":
            process_names = ["google-chrome", "chrome"]
        elif system == "Darwin":
            process_names = ["Google Chrome"]
        else:
            print("不支持的操作系统平台。")
            return
        
        for proc in psutil.process_iter(['pid', 'name']):
            if any(proc.info['name'] == proc_name for proc_name in process_names):
                pid = proc.info['pid']
                process = psutil.Process(pid)
                process.terminate()
                print(f"进程 {pid} 已被终止。")

        print("所有谷歌浏览器程序已关闭。")

        
          




from DrissionPage import ChromiumPage

from colorama import Fore, init
import time


def packet_tree(packet,to_expand='response'):

    def _tree(obj, last_one=True, body='', level=0):
        if obj is None:
            return 1
        
        obj = obj[0] if isinstance(obj, list) and len(obj) !=0 else obj
        if type(obj).__name__=='CaseInsensitiveDict':
            obj=dict(obj)
            pass


        is_dict = isinstance(obj, (dict,))
        not_dict = not is_dict  

        
        show_len = 150
        list_ele = [i for i in dir(obj) if not i.startswith('_')]  if not_dict else obj.keys()

        length = len(list_ele)
        body_unit = '    ' if last_one else '│   '
        tail = '├───'
        new_body = body + body_unit

        if length > 0:
            new_last_one = False
            
            for idx, attr_name in enumerate(list_ele):
                if idx == length - 1:
                    tail = '└───'
                    new_last_one = True
                   
                try:
                    packet_attr = getattr(obj, attr_name) if not_dict else obj[attr_name]
                except:
                    print(f'{attr_name} {obj}  出错！')
                    continue
                
                packet_attr_type = type(packet_attr).__name__
                if    packet_attr_type=='method':
                    continue

                value = str(packet_attr).split('\n')[0][:show_len]  

                # 打印属性信息
                # attr_name=attr_name.ljust(15)
                if packet_attr_type != 'builtin_function_or_method' : 
                        if packet_attr_type=='dict':
                             print(f'{new_body}{tail}< {Fore.BLUE}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  ')
                        else:
                            if is_dict:
                                print(f'{new_body}{tail}# {Fore.GREEN}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  {value}')
                            else:
                                print(f'{new_body}{tail}< {Fore.BLUE}{attr_name}{Fore.RESET} {Fore.RED}{packet_attr_type}{Fore.RESET}  {value}')


      

                # 递归处理特定属性
                if attr_name in fields_to_expand or packet_attr_type in types_to_expand :
                    _tree(packet_attr, new_last_one, new_body, level + 1)
               




    init()
    # fields_to_expand=['response', 'request', 'body', 'headers', 'values','get']
    fields_to_expand=['get']
    fields_to_expand.append(to_expand)
    types_to_expand=['dict','list']

    print(f'{Fore.YELLOW}{packet}{Fore.RESET}')
    _tree(packet)         
            
                    


page = ChromiumPage()
page.listen.start('.jpg')  # 开始监听，指定获取包含该文本的数据包
page.get('https://book.douban.com/')  # 访问网址


i = 0
for packet in page.listen.steps():
    print('\n'*3)  # 打印数据包url    
    packet_tree(packet)
    # print(packet.request.cookies)
    # break
    
    



import pyautogui  as pg
import time
import random
import pygetwindow as gw


def random_float(start, end):
    return round(random.uniform(start, end), 1)

def wait(a,b):
    time.sleep(random_float(a, b))

def three_positive_random_numbers():
    # 生成两个随机数
    num1 = random.random()
    num2 = random.random() * (1 - num1)  # 第二个随机数的范围受第一个随机数影响
    
    # 计算第三个随机数，使得三个数的和为1
    num3 = 1 - num1 - num2
    
    return [num1, num2, num3] 


def random_numbers_list(n):
    numbers = [random.random() for _ in range(n)]
    total = sum(numbers)
    normalized_numbers = [num/total for num in numbers]
    return normalized_numbers


def get_active_window_title():
    active_window = gw.getActiveWindow()
    if active_window is not None:
        return active_window.title
    else:
        return "没有"


def SaoMouseMoveTo(a, b, delay=0.5):
    cishu = 12
    wait(1, 2.5)

    p = pg.position()
    print(p)
    start_x = p.x
    start_y = p.y
    len_x = a - start_x
    len_y = b - start_y
    nodex = [i * len_x for i in random_numbers_list(cishu)]
    nodey = [i * len_y for i in random_numbers_list(cishu)]

    for j in range(cishu):

        pg.moveRel(nodex[j], nodey[j], duration=0.2)
        wait(0.1, 0.2)


def SaoAltTab():
    pg.hotkey('alt', 'tab')
    wait(1,1.5)

def switch_to_window(title):
    i = 1
    while True:
        active_window_title = get_active_window_title()
        print(active_window_title)
        if title in active_window_title:
            break
        j = i
        pg.keyDown('alt')
        for k in range(j):
            pg.press('tab')            
            time.sleep(0.5)
        pg.keyUp('alt')    
        i = i + 1 
        if i == 10:
            break 

def SaoMouseDrag(a=0,b=0,c=100,d=100):
    SaoMouseMoveTo(a,b)
    pg.mouseDown()
    wait(0.1,0.2)
    SaoMouseMoveTo(c,d)
    wait(0.1,0.2)
    pg.mouseUp()        

if __name__ == '__main__':


    switch_to_window("Chrome")

    SaoMouseDrag(802,477,931,476)


# SaoMouseDrag(843,637,993,635,delay=2,duration=4,atctive=True)



from queue import Queue
from DrissionPage._base.driver import Driver
from time import perf_counter, sleep

class Listen_Ws:
    def __init__(self,page) -> None:
        """
        :param page: ChromiumBase对象
        """
        self._page = page
        self._address = page.address
        self._target_id = page._target_id
        self._driver = None

        self._caught = None

        self.listening = False

    def start(self):
        """
        开启websocker监听
        """
        self.clear()
        self._driver = Driver(self._target_id, "page", self._address)
        self._driver.run("Network.enable")
        self._set_callback()
        self.listening = True


    def stop(self):
        """停止监听，清空已监听到的列表"""
        if self.listening:
            self.pause()
            self.clear()
        self._driver.stop()
        self._driver = None

    def pause(self, clear=True):
        """暂停监听
        :param clear: 是否清空已获取队列
        :return: None
        """
        if self.listening:
            self._driver.set_callback('Network.webSocketClosed', None)
            self._driver.set_callback('Network.webSocketCreated', None)
            self._driver.set_callback('Network.webSocketFrameReceived', None)
            self.listening = False
        if clear:
            self.clear()

    def clear(self):
        self._caught = Queue(maxsize=0)

    def steps(self, count=None, timeout=None, gap=1):
        caught = 0
        end = perf_counter() + timeout if timeout else None
        while True:
            if timeout and perf_counter() > end:
                return
            if self._caught.qsize() >= gap:
                yield self._caught.get_nowait() if gap == 1 else [
                    self._caught.get_nowait() for _ in range(gap)
                ]
                if timeout:
                    end = perf_counter() + timeout
                if count:
                    caught += gap
                    if caught >= count:
                        return
            sleep(0.05)

    def _set_callback(self):
        self._driver.set_callback("Network.webSocketClosed", self._websocket_closed)
        self._driver.set_callback("Network.webSocketCreated", self._websocket_created)
        self._driver.set_callback(
            "Network.webSocketFrameReceived", self._websocket_frame_received
        )

    def _websocket_closed(self, **kwargs):
        print("_websocket_closed", kwargs)

    def _websocket_created(self, **kwargs):
        rid = kwargs.get("requestId")

    def _websocket_frame_received(self, **kwargs):
        self._caught.put(kwargs["response"])


import codecs
import os

from setuptools import find_packages, setup

# these things are needed for the README.md show on pypi
# 获取当前文件所在的目录，即项目的根目录
here = os.path.abspath(os.path.dirname(__file__))

# 打开并读取 README.md 文件，使用 utf-8 编码
with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    # 将读取的内容作为包的详细描述，并在前面添加一个空行
    long_description = "\n" + fh.read()



VERSION = '1.0.7'
DESCRIPTION = 'A sao shen library'
LONG_DESCRIPTION = 'The brother version of the Drissionpage library, SaossionPage, is referred to as Sao Shen for short.'

# Setting up
# 使用 setuptools 中的 setup 函数来配置包
setup(
    # 包的名称
    name="SaossionPage",

    # 包的版本号
    version=VERSION,
    # 包的作者
    author="sao shen",
    # 包的作者邮箱
    author_email="",
    # 包的简短描述
    description=DESCRIPTION,
    # 长描述的内容类型
    long_description_content_type="text/markdown",
    # 包的详细描述，从 README.md 文件中读取
    long_description=long_description,
    # 是否包含包中的非 Python 文件
    include_package_data=True,
    # 自动发现和包含所有的包和子包
    packages=find_packages(),
    # 安装这个包所需的依赖项
    # zip_safe=False,
    install_requires=[
        'DrissionPage',
        'colorama',
        
    ],
    # 一些关键词，用于帮助用户在 PyPI 上搜索到这个包
    keywords=['python', 'menu', 'saoshen', 'windows', 'SaossionPage', 'linux'],
    # 一些分类信息，用于描述包的开发状态、适用的受众、编程语言和操作系统等
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


# 1 修改完善 setup.py
# 2 进行本地测试 python setup.py develop
# 3 编译 python setup.py sdist
# 4 上传到pypi twine upload dist/*
